<?php
/*
Plugin Name: NC Sitemap Generator
Description: Generates a sitemap with all pages and posts, updates automatically on adding or deleting entries, and includes a manual update button.
Version: 1.0.0
Author: David Cameselle Martín
*/

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

class NCSitemapGenerator {
    private $sitemap_file;

    public function __construct() {
        $this->sitemap_file = ABSPATH . 'nc-sitemap.xml';
        add_action('save_post', array($this, 'generate_sitemap'));
        add_action('deleted_post', array($this, 'generate_sitemap'));
        
        add_action('admin_menu', array($this, 'add_admin_menu'));
    }

    public function generate_sitemap() {
        $posts = get_posts(array('numberposts' => -1, 'post_type' => array('post', 'page'), 'post_status' => 'publish'));
    
        // Ordenar los posts por tamaño de la URL
        usort($posts, function($a, $b) {
            return strlen(get_permalink($a->ID)) - strlen(get_permalink($b->ID));
        });
    
        $sitemap = '<?xml version="1.0" encoding="UTF-8"?>';
        $sitemap .= '<urlset xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:image="http://www.google.com/schemas/sitemap-image/1.1" xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9 http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd http://www.google.com/schemas/sitemap-image/1.1 http://www.google.com/schemas/sitemap-image/1.1/sitemap-image.xsd" xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">';
    
        // Load existing sitemap if it exists
        $existing_sitemap = array();
        if (file_exists($this->sitemap_file)) {
            $xml = simplexml_load_file($this->sitemap_file);
            if ($xml !== false) {
                foreach ($xml->url as $urlElement) {
                    $existing_sitemap[(string)$urlElement->loc] = array(
                        'changefreq' => (string)$urlElement->changefreq,
                        'priority' => (string)$urlElement->priority
                    );
                }
            }
        }
    
        foreach ($posts as $post) {
            $permalink = get_permalink($post->ID);
            $changefreq = isset($existing_sitemap[$permalink]['changefreq']) ? $existing_sitemap[$permalink]['changefreq'] : 'monthly';
            $priority = isset($existing_sitemap[$permalink]['priority']) ? $existing_sitemap[$permalink]['priority'] : '0.9';
    
            $sitemap .= '<url>';
            $sitemap .= '<loc>' . $permalink . '</loc>';
            $sitemap .= '<lastmod>' . get_the_modified_time('c', $post->ID) . '</lastmod>';
            $sitemap .= '<changefreq>' . $changefreq . '</changefreq>';
            $sitemap .= '<priority>' . $priority . '</priority>';
            $sitemap .= '</url>';
        }
    
        $sitemap .= '</urlset>';
    
        file_put_contents($this->sitemap_file, $sitemap);
    }

    public function add_admin_menu() {
        add_menu_page(
            'NyarCode Plugins',
            'NyarCode',
            'manage_options',
            'nyarcode',
            '',
            'dashicons-admin-generic',
            6
        );

        add_submenu_page(
            'nyarcode',
            'NC Sitemap Generator',
            'Sitemap Generator',
            'manage_options',
            'nc-sitemap-generator',
            array($this, 'admin_page')
        );

        remove_submenu_page('nyarcode', 'nyarcode');
    }

    public function admin_page() {
        if (isset($_POST['generate_sitemap'])) {
            $this->generate_sitemap();
            echo '<div class="updated"><p>Sitemap generated successfully!</p></div>';
        }
    
        if (isset($_POST['update_changefreq'])) {
            $this->update_changefreq($_POST['url'], $_POST['changefreq']);
            echo '<div class="updated"><p>Change frequency updated successfully!</p></div>';
        }

        if (isset($_POST['update_priority'])) {
            $this->update_priority($_POST['url'], $_POST['priority']);
            echo '<div class="updated"><p>Priority updated successfully!</p></div>';
        }
        ?>
        <div class="wrap">
            <h1>NC Sitemap Generator</h1>
            <form method="post">
            <input type="submit" name="generate_sitemap" class="button button-primary" value="Generate Sitemap">
            </form>
            <h2><small>Sitemap created by NyarCode</small></h2>
            <a href="<?php echo home_url('/nc-sitemap.xml'); ?>" target="_blank">View Sitemap</a>
            <hr>
            <?php if (file_exists($this->sitemap_file)) : ?>
            <?php
            $xml = simplexml_load_file($this->sitemap_file);
            if ($xml !== false) :
            ?>
                <table class="widefat fixed" cellspacing="0">
                <thead>
                    <tr>
                    <th>URL</th>
                    <th>Change Freq</th>
                    <th>Priority</th>
                    <th>Last Modified</th>
                    </tr>
                </thead>
                <tbody>
                    <?php foreach ($xml->url as $url) : ?>
                    <tr>
                        <td><?php echo esc_html($url->loc); ?></td>
                        <td>
                            <form method="post">
                                <input type="hidden" name="url" value="<?php echo esc_attr($url->loc); ?>">
                                <select name="changefreq" onchange="this.form.submit()">
                                    <option value="daily" <?php selected($url->changefreq, 'daily'); ?>>Daily</option>
                                    <option value="weekly" <?php selected($url->changefreq, 'weekly'); ?>>Weekly</option>
                                    <option value="monthly" <?php selected($url->changefreq, 'monthly'); ?>>Monthly</option>
                                    <option value="yearly" <?php selected($url->changefreq, 'yearly'); ?>>Yearly</option>
                                </select>
                                <input type="hidden" name="update_changefreq" value="1">
                            </form>
                        </td>
                        <td>
                            <form method="post">
                                <input type="hidden" name="url" value="<?php echo esc_attr($url->loc); ?>">
                                <input type="number" step="0.1" min="0.0" max="1.0" name="priority" value="<?php echo esc_attr($url->priority); ?>" onchange="this.form.submit()">
                                <input type="hidden" name="update_priority" value="1">
                            </form>
                        </td>
                        <td><?php echo esc_html($url->lastmod); ?></td>
                    </tr>
                    <?php endforeach; ?>
                </tbody>
                </table>
            <?php else : ?>
                <p>Failed to load sitemap.xml</p>
            <?php endif; ?>
            <?php else : ?>
            <p>Sitemap not generated yet.</p>
            <?php endif; ?>
        </div>
        <?php
    }
    
    private function update_changefreq($url, $changefreq) {
        $xml = simplexml_load_file($this->sitemap_file);
        if ($xml !== false) {
            foreach ($xml->url as $urlElement) {
                if ((string)$urlElement->loc == $url) {
                    $urlElement->changefreq = $changefreq;
                    break;
                }
            }
            $xml->asXML($this->sitemap_file);
        }
    }

    private function update_priority($url, $priority) {
        // Ensure priority is between 0 and 1
        $priority = max(0, min(1, floatval($priority)));
    
        $xml = simplexml_load_file($this->sitemap_file);
        if ($xml !== false) {
            foreach ($xml->url as $urlElement) {
                if ((string)$urlElement->loc == $url) {
                    $urlElement->priority = $priority;
                    break;
                }
            }
            $xml->asXML($this->sitemap_file);
        }
    }
}

new NCSitemapGenerator();